/*
    Qalculate    

    Copyright (C) 2003  Niklas Knutsson (nq@altern.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <unistd.h>
#include "support.h"

#include "data/icon.xpm"

#include <libqalculate/qalculate.h>

GdkPixbuf *icon_pixbuf;
GtkWidget *nbases_dialog;
GtkWidget *nbases_entry_binary;
GtkWidget *nbases_entry_octal;
GtkWidget *nbases_entry_hexadecimal;
GtkWidget *nbases_label_decimal;
GtkWidget *nbases_label_octal;
GtkWidget *nbases_label_hexadecimal;
GtkWidget *nbases_label_binary;
GtkWidget *nbases_entry_decimal;
GtkWidget *nbases_button_close;

bool changing_in_nbases_dialog = false;	

#ifdef __cplusplus
extern "C" {
#endif

void on_nbases_button_close_clicked(GtkButton *button, gpointer user_data);
void on_nbases_entry_decimal_changed(GtkEditable *editable, gpointer user_data);
void on_nbases_entry_binary_changed(GtkEditable *editable, gpointer user_data);
void on_nbases_entry_octal_changed(GtkEditable *editable, gpointer user_data);
void on_nbases_entry_hexadecimal_changed(GtkEditable *editable, gpointer user_data);
void close_app();

void update_nbases_entries(const MathStructure &value, int base) {
	g_signal_handlers_block_matched((gpointer) nbases_entry_decimal, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, (gpointer) on_nbases_entry_decimal_changed, NULL);			
	g_signal_handlers_block_matched((gpointer) nbases_entry_binary, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, (gpointer) on_nbases_entry_binary_changed, NULL);
	g_signal_handlers_block_matched((gpointer) nbases_entry_octal, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, (gpointer) on_nbases_entry_octal_changed, NULL);
	g_signal_handlers_block_matched((gpointer) nbases_entry_hexadecimal, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, (gpointer) on_nbases_entry_hexadecimal_changed, NULL);
	PrintOptions po;
	po.number_fraction_format = FRACTION_DECIMAL;
	po.lower_case_e = true;
	if(base != 10) {po.base = 10; gtk_entry_set_text(GTK_ENTRY(nbases_entry_decimal), value.print(po).c_str());}
	if(base != 2) {po.base = 2; gtk_entry_set_text(GTK_ENTRY(nbases_entry_binary), value.print(po).c_str());}	
	if(base != 8) {po.base = 8; gtk_entry_set_text(GTK_ENTRY(nbases_entry_octal), value.print(po).c_str());}	
	if(base != 16) {po.base = 16; gtk_entry_set_text(GTK_ENTRY(nbases_entry_hexadecimal), value.print(po).c_str());}	
	g_signal_handlers_unblock_matched((gpointer) nbases_entry_decimal, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, (gpointer) on_nbases_entry_decimal_changed, NULL);			
	g_signal_handlers_unblock_matched((gpointer) nbases_entry_binary, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, (gpointer) on_nbases_entry_binary_changed, NULL);
	g_signal_handlers_unblock_matched((gpointer) nbases_entry_octal, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, (gpointer) on_nbases_entry_octal_changed, NULL);
	g_signal_handlers_unblock_matched((gpointer) nbases_entry_hexadecimal, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, (gpointer) on_nbases_entry_hexadecimal_changed, NULL);
}
void on_nbases_button_close_clicked(GtkButton*, gpointer) {
	close_app();
}
void close_app() {
	gtk_main_quit();
}
void on_nbases_entry_decimal_changed(GtkEditable *editable, gpointer) {
	if(changing_in_nbases_dialog) return;
	string str = gtk_entry_get_text(GTK_ENTRY(editable));
	remove_blank_ends(str);
	if(str.empty()) return;
	if(is_in(OPERATORS EXP, str[str.length() - 1])) return;
	EvaluationOptions eo;
	eo.structuring = STRUCTURING_NONE;
	eo.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
	eo.parse_options.dot_as_separator = CALCULATOR->default_dot_as_separator;
	changing_in_nbases_dialog = true;
	update_nbases_entries(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(gtk_entry_get_text(GTK_ENTRY(editable))), eo), 10);
	changing_in_nbases_dialog = false;	
}
void on_nbases_entry_binary_changed(GtkEditable *editable, gpointer) {
	if(changing_in_nbases_dialog) return;
	string str = gtk_entry_get_text(GTK_ENTRY(editable));
	remove_blank_ends(str);
	if(str.empty()) return;
	if(is_in(OPERATORS, str[str.length() - 1])) return;
	EvaluationOptions eo;
	eo.structuring = STRUCTURING_NONE;
	eo.parse_options.base = BASE_BINARY;
	eo.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
	eo.parse_options.dot_as_separator = CALCULATOR->default_dot_as_separator;
	changing_in_nbases_dialog = true;	
	update_nbases_entries(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(str), eo), 2);
	changing_in_nbases_dialog = false;	
}
void on_nbases_entry_octal_changed(GtkEditable *editable, gpointer) {
	if(changing_in_nbases_dialog) return;
	string str = gtk_entry_get_text(GTK_ENTRY(editable));
	remove_blank_ends(str);
	if(str.empty()) return;	
	if(is_in(OPERATORS, str[str.length() - 1])) return;
	EvaluationOptions eo;
	eo.structuring = STRUCTURING_NONE;
	eo.parse_options.base = BASE_OCTAL;
	eo.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
	eo.parse_options.dot_as_separator = CALCULATOR->default_dot_as_separator;
	changing_in_nbases_dialog = true;	
	update_nbases_entries(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(str), eo), 8);
	changing_in_nbases_dialog = false;	
}
void on_nbases_entry_hexadecimal_changed(GtkEditable *editable, gpointer) {
	if(changing_in_nbases_dialog) return;
	string str = gtk_entry_get_text(GTK_ENTRY(editable));
	remove_blank_ends(str);
	if(str.empty()) return;	
	if(is_in(OPERATORS, str[str.length() - 1])) return;
	EvaluationOptions eo;
	eo.structuring = STRUCTURING_NONE;
	eo.parse_options.base = BASE_HEXADECIMAL;
	eo.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
	eo.parse_options.dot_as_separator = CALCULATOR->default_dot_as_separator;
	changing_in_nbases_dialog = true;	
	update_nbases_entries(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(str), eo), 16);
	changing_in_nbases_dialog = false;	
}

#ifdef __cplusplus
}
#endif

GtkWidget* create_nbases_dialog(void) {

	GtkWidget *dialog_vbox5;
	GtkWidget *table4;
	GtkWidget *dialog_action_area5;
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new ();

	nbases_dialog = gtk_dialog_new ();
	gtk_container_set_border_width (GTK_CONTAINER (nbases_dialog), 5);
	gtk_window_set_title (GTK_WINDOW (nbases_dialog), _("Number Bases"));
	gtk_window_set_resizable (GTK_WINDOW (nbases_dialog), FALSE);
	gtk_window_set_type_hint (GTK_WINDOW (nbases_dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_dialog_set_has_separator (GTK_DIALOG (nbases_dialog), FALSE);

	dialog_vbox5 = GTK_DIALOG (nbases_dialog)->vbox;
	gtk_widget_show (dialog_vbox5);

	table4 = gtk_table_new (4, 2, TRUE);
	gtk_widget_show (table4);
	gtk_box_pack_start (GTK_BOX (dialog_vbox5), table4, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (table4), 12);
	gtk_table_set_row_spacings (GTK_TABLE (table4), 6);
	gtk_table_set_col_spacings (GTK_TABLE (table4), 6);

	nbases_entry_binary = gtk_entry_new ();
	gtk_entry_set_alignment(GTK_ENTRY(nbases_entry_binary), 1.0);
	gtk_widget_show (nbases_entry_binary);
	gtk_table_attach (GTK_TABLE (table4), nbases_entry_binary, 1, 2, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_tooltips_set_tip (tooltips, nbases_entry_binary, _("Binary value"), NULL);

	nbases_entry_octal = gtk_entry_new ();
	gtk_entry_set_alignment(GTK_ENTRY(nbases_entry_octal), 1.0);
	gtk_widget_show (nbases_entry_octal);
	gtk_table_attach (GTK_TABLE (table4), nbases_entry_octal, 1, 2, 2, 3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_tooltips_set_tip (tooltips, nbases_entry_octal, _("Octal value"), NULL);

	nbases_entry_hexadecimal = gtk_entry_new ();
	gtk_entry_set_alignment(GTK_ENTRY(nbases_entry_hexadecimal), 1.0);
	gtk_widget_show (nbases_entry_hexadecimal);
	gtk_table_attach (GTK_TABLE (table4), nbases_entry_hexadecimal, 1, 2, 3, 4, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_tooltips_set_tip (tooltips, nbases_entry_hexadecimal, _("Hexadecimal value"), NULL);

	nbases_label_decimal = gtk_label_new (_("Decimal"));
	gtk_widget_show (nbases_label_decimal);
	gtk_table_attach (GTK_TABLE (table4), nbases_label_decimal, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (nbases_label_decimal), 0, 0.5);

	nbases_label_octal = gtk_label_new (_("Octal"));
	gtk_widget_show (nbases_label_octal);
	gtk_table_attach (GTK_TABLE (table4), nbases_label_octal, 0, 1, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (nbases_label_octal), 0, 0.5);

	nbases_label_hexadecimal = gtk_label_new (_("Hexadecimal"));
	gtk_widget_show (nbases_label_hexadecimal);
	gtk_table_attach (GTK_TABLE (table4), nbases_label_hexadecimal, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (nbases_label_hexadecimal), 0, 0.5);

	nbases_label_binary = gtk_label_new (_("Binary"));
	gtk_widget_show (nbases_label_binary);
	gtk_table_attach (GTK_TABLE (table4), nbases_label_binary, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0); 
	gtk_misc_set_alignment (GTK_MISC (nbases_label_binary), 0, 0.5);

	nbases_entry_decimal = gtk_entry_new ();
	gtk_entry_set_alignment(GTK_ENTRY(nbases_entry_decimal), 1.0);
	gtk_widget_show (nbases_entry_decimal);
	gtk_table_attach (GTK_TABLE (table4), nbases_entry_decimal, 1, 2, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_tooltips_set_tip (tooltips, nbases_entry_decimal, _("Decimal value"), NULL);

	dialog_action_area5 = GTK_DIALOG (nbases_dialog)->action_area;
	gtk_widget_show (dialog_action_area5);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area5), GTK_BUTTONBOX_END);

	nbases_button_close = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (nbases_button_close);
	gtk_dialog_add_action_widget (GTK_DIALOG (nbases_dialog), nbases_button_close, GTK_RESPONSE_CLOSE);
	GTK_WIDGET_SET_FLAGS (nbases_button_close, GTK_CAN_DEFAULT);
	gtk_tooltips_set_tip (tooltips, nbases_button_close, _("Close this window"), NULL);

	icon_pixbuf = gdk_pixbuf_new_from_xpm_data((const char**) icon_xpm);
	gtk_window_set_icon(GTK_WINDOW(nbases_dialog), icon_pixbuf);
	
	gtk_widget_grab_focus(nbases_entry_decimal);

	g_signal_connect_swapped ((gpointer) nbases_dialog, "delete_event", G_CALLBACK (close_app), NULL);
	g_signal_connect_swapped ((gpointer) nbases_dialog, "destroy_event", G_CALLBACK (close_app), NULL);
	g_signal_connect ((gpointer) nbases_entry_binary, "changed", G_CALLBACK (on_nbases_entry_binary_changed), NULL);
	g_signal_connect ((gpointer) nbases_entry_octal, "changed", G_CALLBACK (on_nbases_entry_octal_changed), NULL);
	g_signal_connect ((gpointer) nbases_entry_hexadecimal, "changed", G_CALLBACK (on_nbases_entry_hexadecimal_changed), NULL);
	g_signal_connect ((gpointer) nbases_entry_decimal, "changed", G_CALLBACK (on_nbases_entry_decimal_changed), NULL);
	g_signal_connect ((gpointer) nbases_button_close, "clicked", G_CALLBACK (on_nbases_button_close_clicked), NULL);

	return nbases_dialog;
  
}



int main (int argc, char **argv) {

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	gtk_init(&argc, &argv);

	//create the almighty Calculator object
	new Calculator();

	create_nbases_dialog();	
	gtk_widget_show(nbases_dialog);
	
	gtk_main();
	
	return 0;
}
